# -*- coding: UTF-8 -*-

import sys
from sqlUnloader import SQLUnloader, CaseInsensitiveDict
from common import logger
import ast

class SecurityLogUnloader(SQLUnloader):
    def __init__(self, connection, log):
        self.cursor = connection.cursor()
        self.sdata = connection.cursor()
        self.log = log
    
    def getExtrainfo(self, commandname):
        try:
            return ast.literal_eval(commandname)
        except:
            pass
        return {}

    def unloadSecurityLogObjects(self):
        for row in self.sdata:
            dict_lower = CaseInsensitiveDict(row)
            yield {'crightsid': self.getValue(dict_lower, 'crights.crightsid'),
                    'shiftnum': self.getValue(dict_lower, 'workshift.shiftnum'),
                    'cashiercard': self.getValue(dict_lower, 'crights.cashiercard'),
                    'cashiername': self.getValue(dict_lower, 'crights.cashiername'),
                    'actorcard': self.getValue(dict_lower, 'crights.actorcard'),
                    'actorname': self.getValue(dict_lower, 'crights.actorname'),
                    'op': self.getValue(dict_lower, 'crights.op'),
                    'isgoodsitem': self.getValue(dict_lower, 'crights.isgoodsitem'),
                    'optime': self.getValue(dict_lower, 'crights.optime'),
                    'opcode': self.getValue(dict_lower, 'crights.opcode'),
                    'commandname': self.getValue(dict_lower, 'crights.commandname'),
                    'cashcode': self.getValue(dict_lower, 'crights.cashcode'),
                    'shifttype': self.getValue(dict_lower, 'workshift.shifttype') if self.getValue(dict_lower, 'workshift.shifttype') else 0,
                    'extrainfo': self.getExtrainfo(self.getValue(dict_lower, 'crights.commandname'))
                   }

    def unloadSecurityLogs(self, shiftIds):
        if len(shiftIds) > 2:
            self.log.info('unload securityLogs for shiftIds %s' % (shiftIds))
            query = "select * from crights, workshift where crights.workshiftid IN %s and crights.workshiftid=workshift.workshiftid" % shiftIds
            self.sdata.execute(query)
            for o in self.unloadSecurityLogObjects():
                yield o
        else:
            self.log.info(u'Журнал выполненных операций пуст')

    def unloadSecurityLogsByDate(self, shiftIds, fromDate, toDate):
        if len(shiftIds) > 2:
            self.log.info('unload securityLogs from %s to %s' % (str(fromDate), str(toDate)))
            self.sdata.execute("select * from crights, workshift where crights.workshiftid IN %s and crights.workshiftid=workshift.workshiftid \
                                and crights.optime > '%s' and crights.optime <= '%s'" % (shiftIds, fromDate, toDate))
            for o in self.unloadSecurityLogObjects():
                yield o

    def unloadSecurityLogsByDocuments(self, docIds):
        if len(docIds) > 2:
            self.log.info('unload securityLogs for docIds %s' % (docIds))

            self.cursor.execute('SELECT DISTINCT(workshiftid) FROM document WHERE documentid IN %s' % docIds)
            shiftIds = []
            for shiftId in self.cursor.fetchall():
                self.log.info('shiftId = %s' % shiftId)
                shiftIds.append(int(shiftId.get('workshiftid')))
            shiftIds = '('+str(shiftIds)[1:-1]+')'
            self.cursor.execute('SELECT MIN(time_beg) FROM document WHERE documentid IN %s' % docIds)
            fromDate = self.cursor.fetchone().get('MIN(time_beg)')
            self.cursor.execute('SELECT MAX(time_end) FROM document WHERE documentid IN %s' % docIds)
            toDate = self.cursor.fetchone().get('MAX(time_end)')

            return self.unloadSecurityLogsByDate(shiftIds, fromDate, toDate)
        else:
            self.log.info('Documents not found')
            return ()
