# -*- coding: UTF-8 -*-

import sys
from sqlUnloader import SQLUnloader, CaseInsensitiveDict
from common import logger

class ShiftUnloader(SQLUnloader):
    def __init__(self, connection, log, storeId):
        self.cursor = connection.cursor()
        self.sdata = connection.cursor()
        self.udata = connection.cursor()
        self.log = log
        self.storeId = storeId

    def unloadShiftObject(self, dict_lower):
        return {'cashCode': self.getValue(dict_lower, 'workshift.cashcode'),
                'shift': self.getValue(dict_lower, 'workshift.shiftnum'),
                'userCode': self.getValue(dict_lower, 'workshift.scode'),
                'timeBeg': self.getValue(dict_lower, 'workshift.time_beg'),
                'timeEnd': self.getValue(dict_lower, 'workshift.time_end'),
                'checkNum1': self.getValue(dict_lower, 'workshift.checknum1'),
                'checkNum2': self.getValue(dict_lower, 'workshift.checknum2'),
                'sumSale': self.getValue(dict_lower, 'workshift.sumsale'),
                'sumGain': self.getValue(dict_lower, 'workshift.sumgain'),
                'type': self.getValue(dict_lower, 'workshift.shifttype'),
                'shopcode': self.getValue(dict_lower, 'workshift.shopcode'),
                'sumDrawer': self.getValue(dict_lower, 'workshift.sumDrawer'),
                'update_time': self.getValue(dict_lower, 'workshift.update_time'),
                'users': [],
                'firstchecktime': self.getValue(dict_lower, 'workshift.firstchecktime')
                }
    
    def unloadUserObject(self, dict_lower):
        return {'usercode': self.getValue(dict_lower, 'users.usercode'), 
                'username': self.getValue(dict_lower, 'users.username'),
                'rank': self.getValue(dict_lower, 'users.rank')}
        
    def unloadKKMObject(self, dict_lower):
        kkms = {'kkmNum': self.getValue(dict_lower, 'kkm.kkmnum'),
                'producerCode': self.getValue(dict_lower, 'kkm.producercode'),
                'producerName': self.getValue(dict_lower, 'kkm.producername'),
                'modelNum': self.getValue(dict_lower, 'kkm.modelnum'),
                'modelName': self.getValue(dict_lower, 'kkm.modelname'),
                'serialNum': self.getValue(dict_lower, 'kkm.serialnum'),
                'sumProtectedBeg': self.getValue(dict_lower, 'kkm.sumprotectedbeg'),
                'sumProtectedEnd': self.getValue(dict_lower, 'kkm.sumprotectedend'),
                'sumCashBeg': self.getValue(dict_lower, 'kkm.sumcashbeg'),
                'sumCashEnd': self.getValue(dict_lower, 'kkm.sumcashend'),
                'sumGain': self.getValue(dict_lower, 'kkm.sumgain'),
                'countSale': self.getValue(dict_lower, 'kkm.countsale'),
                'sumSale': self.getValue(dict_lower, 'kkm.sumsale'),
                'sumSaleEklz': self.getValue(dict_lower, 'kkm.sumsaleeklz'),
                'sumSaleByPayment1': self.getValue(dict_lower, 'kkm.sumsalebypayment1'),
                'sumSaleByPayment2': self.getValue(dict_lower, 'kkm.sumsalebypayment2'),
                'sumSaleByPayment3': self.getValue(dict_lower, 'kkm.sumsalebypayment3'),
                'sumSaleByPayment4': self.getValue(dict_lower, 'kkm.sumsalebypayment4'),
                'sumSaleByPayment5': self.getValue(dict_lower, 'kkm.sumsalebypayment5'),
                'sumSaleByPayment6': self.getValue(dict_lower, 'kkm.sumsalebypayment6'),
                'countBack': self.getValue(dict_lower, 'kkm.countback'),
                'sumBack': self.getValue(dict_lower, 'kkm.sumback'),
                'sumBackEklz': self.getValue(dict_lower, 'kkm.sumbackeklz'),
                'sumBackByPayment1': self.getValue(dict_lower, 'kkm.sumbackbypayment1'),
                'sumBackByPayment2': self.getValue(dict_lower, 'kkm.sumbackbypayment2'),
                'sumBackByPayment3': self.getValue(dict_lower, 'kkm.sumbackbypayment3'),
                'sumBackByPayment4': self.getValue(dict_lower, 'kkm.sumbackbypayment4'),
                'sumBackByPayment5': self.getValue(dict_lower, 'kkm.sumbackbypayment5'),
                'sumBackByPayment6': self.getValue(dict_lower, 'kkm.sumbackbypayment6'),
                'paymentName1': self.getValue(dict_lower, 'kkm.paymentname1'),
                'paymentName2': self.getValue(dict_lower, 'kkm.paymentname2'),
                'paymentName3': self.getValue(dict_lower, 'kkm.paymentname3'),
                'paymentName4': self.getValue(dict_lower, 'kkm.paymentname4'),
                'paymentName5': self.getValue(dict_lower, 'kkm.paymentname5'),
                'paymentName6': self.getValue(dict_lower, 'kkm.paymentname6'),
                'sumMoneyIn': self.getValue(dict_lower, 'kkm.summoneyin'),
                'countMoneyIn': self.getValue(dict_lower, 'kkm.countmoneyin'),
                'sumMoneyOut': self.getValue(dict_lower, 'kkm.summoneyout'),
                'countMoneyOut': self.getValue(dict_lower, 'kkm.countmoneyout'),
                'crashed': self.getValue(dict_lower, 'kkm.crashed'),
                'shiftnumkkm': self.getValue(dict_lower, 'kkm.shiftnumkkm'),
                'unsenddoccount': self.getValue(dict_lower, 'kkm.unsenddoccount'),
                'unsenddocdate': self.getValue(dict_lower, 'kkm.unsenddocdate'),
                'fnnumber': self.getValue(dict_lower, 'kkm.fnnumber'),
                }
        for columnName in ['sumsaledept', 'sumbackdept']:
            for num in range(1, 17):
                kkms['%s%s' % (columnName, num)] = self.getValue(dict_lower, 'kkm.%s%s' % (columnName, num))
        return kkms

    def getClosedShiftsQueryEnd(self, closedShifts):
        queryEnd = ""
        if closedShifts == 0:
            queryEnd = " and s.time_end IS NOT NULL"
        elif closedShifts == 1:
            queryEnd = " and s.time_end IS NULL"
        return queryEnd
    
    def getStoreIdQueryPart(self):
        if logger.Logger().location == 'cs':
            return " AND s.storeId = '%s' " % self.storeId
        return ''
        
    def getChangedShifts(self, cashCode, closedShifts):
        self.log.info('get changed shifts for cash %s' % str(cashCode))
        queryEnd = self.getClosedShiftsQueryEnd(closedShifts)
        query = "select workshiftid from workshift s where s.cashcode='%s' and s.changed=1%s%s" % (cashCode, self.getStoreIdQueryPart(), queryEnd)
        self.cursor.execute(query)
        return self.cursor.fetchall()

    def getShiftsByDate(self, cashCode, fromDate, toDate, closedShifts):
        self.log.info('get shifts by date from %s to %s for cash %s' % (str(fromDate), str(toDate), cashCode))
        queryEnd = self.getClosedShiftsQueryEnd(closedShifts)
        query = "select workshiftid from workshift s where s.cashcode = '%(cashCode)s' %(storeId)s AND (('%(fromDate)s' <= time_beg AND time_beg <= '%(toDate)s') OR \
                  ('%(fromDate)s' <= time_end AND time_end <= '%(toDate)s') OR (time_beg <= '%(fromDate)s' AND ('%(toDate)s' <= time_end OR time_end IS NULL)))\
                  %(queryEnd)s" % {'cashCode':cashCode, 'storeId':self.getStoreIdQueryPart(), 'fromDate':fromDate, 'toDate':toDate, 'queryEnd':queryEnd}
        self.cursor.execute(query)
        return self.cursor.fetchall()

    def getShiftsByShiftnum(self, cashCode, fromShift, toShift, closedShifts):
        self.log.info('get shifts by number from %s to %s for cash %s' % (str(fromShift), str(toShift), cashCode))
        queryEnd = self.getClosedShiftsQueryEnd(closedShifts)
        query = "select workshiftid from workshift s where cashcode = '%s' and shiftnum >= %s and shiftnum <= %s%s%s" % (cashCode, fromShift, toShift, self.getStoreIdQueryPart(), queryEnd)
        self.cursor.execute(query)
        return self.cursor.fetchall()

    def getShiftsByDocumentDate(self, cashCode, fromDate, toDate):
        self.log.info('get shifts by documents date from %s to %s for cash %s' % (str(fromDate), str(toDate), cashCode))

        query = "select workshiftid from workshift s where cashcode = '%s' %s AND (workshiftid IN \
                            (SELECT DISTINCT workshiftid FROM document WHERE time_end > '%s' AND time_end <= '%s') OR \
                            (time_end > '%s' AND time_end <= '%s'))" % (cashCode, self.getStoreIdQueryPart(), fromDate, toDate, fromDate, toDate)
        self.cursor.execute(query)
        return self.cursor.fetchall()

    def unloadUsers(self, shiftid):
        query = "select * from users where workshiftid = %s"
        self.udata.execute(query, (shiftid,))
        for row in self.udata:
            dict_lower = CaseInsensitiveDict(row)
            yield self.unloadUserObject(dict_lower)

    def unloadKKMs(self, shiftid):
        query = "select * from kkm where workshiftid = %s"
        self.udata.execute(query, (shiftid,))
        for row in self.udata:
            dict_lower = CaseInsensitiveDict(row)
            yield self.unloadKKMObject(dict_lower)

    def unloadShifts(self, shifts):
        if len(shifts) > 2:
            self.log.info('unload shifts %s' % (shifts))
            query = "select * from workshift where workshiftid IN %s" % shifts
            self.sdata.execute(query)
            for row in self.sdata:
                dict_lower = CaseInsensitiveDict(row)
                shift = self.unloadShiftObject(dict_lower)
                workshiftid = self.getValue(dict_lower, 'workshiftid')
                shift['users'] = [pos for pos in self.unloadUsers(workshiftid)]
                shift['kkms'] = [pos for pos in self.unloadKKMs(workshiftid)]
                yield shift
        else:
            self.log.info(u'Нет смен за указанный период')

    def getShiftsByDocumentDateForShop(self, fromDate, toDate, cashList):
        self.log.info('get shifts by documents date from %s to %s for all cash ' % (str(fromDate), str(toDate)))

        query = "select workshiftid from workshift s where cashcode IN %s %s AND (workshiftid IN \
                            (SELECT DISTINCT workshiftid FROM document WHERE time_end > '%s' AND time_end <= '%s') OR \
                            (time_end > '%s' AND time_end <= '%s'))" % (cashList, self.getStoreIdQueryPart(), fromDate, toDate, fromDate, toDate)
        self.cursor.execute(query)
        return self.cursor.fetchall()

    def getShiftsByShiftsForShop(self, fromShift, toShift, cashList, closedShifts):
        self.log.info(u'Получение смен с номера %s по %s для касс (%s)' % (str(fromShift), str(toShift), str((cashList))[1:-1]))
        queryEnd = self.getClosedShiftsQueryEnd(closedShifts)
        query = "select workshiftid from workshift s where cashcode IN %s and shiftnum >= %s and shiftnum <= %s%s%s" \
                % ('('+str((cashList))[1:-1]+')', fromShift, toShift, self.getStoreIdQueryPart(), queryEnd)

        self.cursor.execute(query)
        return self.cursor.fetchall()

    def getOpenedShiftsToDay(self, cashCode):
        self.log.info(u'Получение открытых смен за сегодня для кассы №%s' % cashCode)
        query = "SELECT workshiftid FROM workshift WHERE cashcode = %s AND (DATE(time_beg) = CURDATE() AND (DATE(time_end) = CURDATE() OR time_end IS NULL) OR (time_beg IS NULL and time_end IS NULL));"
        self.cursor.execute(query, (cashCode,))
        return self.cursor.fetchall()
