# -*- coding: UTF-8 -*-

import json
import ujson
import optparse
import sys, os
from datetime import datetime
if os.path.split(sys.path[0])[1] == 'converter':
    sys.path[0] = os.path.normpath('%s/../..' % sys.path[0])
    sys.path.append(os.path.normpath('%s/upload/node' % sys.path[0]))

from common import filesmodify, logger, config, macros, aif_objects
from upload.inserter import InserterError
maxCountBackupFiles = None
actualCountDaysForBackupFiles = None
errorObjectsCount = 0

def filterKeys(d):
    lowerD = {}
    for k in d.keys():
        if d[k] != None and d[k] != []:
            lowerD[k.lower()] = d[k]
    return lowerD

def countOjbects(useQueue, inserter, data, log):
    '''
    Подсчитывает количество объектов для загрузки в БД
    Используется при настройке - запись в БД напрямую без очереди
    Записывает данные в /linuxcash/cash/exchangesystems/progress/counter_%s.json 
    '''
    if useQueue:
        return

    messId = hex(hash(datetime.now()))
    inserter.startMessageId = messId
    cntObjects = 0
    for dictFile in data:
        cntObjects += sum(1 for line in open(dictFile).xreadlines() if line.strip() == "---")
    f = open('/linuxcash/cash/exchangesystems/progress/counter_%s.json' % messId,'w')
    log.info(u'Количество загружаемых объектов: %s' % cntObjects)
    f.write('{"objectsCount": %s}\n' % cntObjects)
    f.close()

def jsonWrapper(jsonPiece):
    '''
    Обертка для ф-ии json.loads
    '''
    return json.loads(jsonPiece, object_hook = filterKeys)

def invalidObjectsCount():
    return errorObjectsCount

def general_method(workDirectory, data, node, useQueue = True, extendedProperties = {}):
    global errorObjectsCount
    hotKeyByBarcode = node.currentConverter.hotKeyByBarcode
    errorList = []
    log = logger.Logger().getLoggerUpload(node.id)
    currentObject = None
    try:
        plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
        cl = getattr(plugin, 'Inserter')
        inserter = cl(node)
        result = False
        listObjects = []
        if data:
            countOjbects(useQueue, inserter, data, log)
        result = inserter.start()
        if result and data:
            for dictFile in data:
                dictFile = open(dictFile, 'rb')
                log.info(u'Отправляем объекты справочника в %s...' % (u'очередь' if useQueue else u'БД'))
                jsonPiece=''
                firstLineCmd = True
                endMarker = False
                beginMarker = False
                jloads = ujson.loads if extendedProperties.get('version', 1) >= '2' else jsonWrapper
                for line in dictFile.xreadlines():
                    if not len(listObjects) and line.strip() == "### data begin ###":
                        log.info(u"Найден маркер начала справочника")
                        beginMarker = True
                    elif beginMarker and line.strip() == "### data end ###":
                        log.info(u"Найден маркер конца справочника")
                        endMarker = True
                    elif line.strip() == "---":
                        if jsonPiece.strip() != '':
                            currentObject = jsonPiece
                            dictObject = jloads(jsonPiece)
                            if hotKeyByBarcode:
                                if 'addHotKey' == dictObject['command'] and not 'bybarcode' in dictObject['hotkey']:
                                    dictObject['hotkey']['bybarcode'] = hotKeyByBarcode
                            listObjects.append(dictObject)
                        jsonPiece=''
                    else:
                        if firstLineCmd:
                            startPos = line.find('{')
                            if startPos != -1:
                                firstLineCmd = False
                                line = line[startPos:]
                            else:
                                line = ''
                        jsonPiece+=' '+line
                    if len(listObjects) == node.pack_size:
                        inserter.add_package(listObjects)
                        listObjects = []

                if beginMarker and not endMarker:
                    currentObject = None
                    raise Exception(u'Не найден маркер конца справочника')

                if len(listObjects) > 0:
                    inserter.add_package(listObjects)
                    listObjects = []
                dictFile.close()
            result = inserter.stop()
            errorObjectsCount = inserter.invalidObjectsCount()
        if result and not data:
            result = inserter.stop()
        return result, errorList
    except InserterError, e:
        raise
    except aif_objects.AifObjectNotValid, e:
        log.error(u'Ошибка: %s' % e)
        errorList.append(u'Ошибка: %s' % e)
    except Exception, e:
        log.error(u'Ошибка: %s' % e)
        if currentObject:
            log.error(u'Ошибка произошла на объекте: %s' % unicode(currentObject, 'utf-8'))
            errorList.append(u'Ошибка произошла на объекте: %s' % unicode(currentObject, 'utf-8'))
        else:
            errorList.append(u'Ошибка: %s' % e)
    return False, errorList


def convert(workDirectory, node, useQueue = True, extendedProperties = {}):
    workDirectory = os.path.normpath('%s/%s/aif' % (workDirectory, node.id))
    if not os.path.exists(workDirectory):
        os.makedirs(workDirectory)
    if node.currentConverter.flags:
        filesmodify.moveFiles(node.currentConverter.flags, workDirectory)
    data = filesmodify.moveFiles(node.currentConverter.data, workDirectory) if node.currentConverter.data else None
    result, errorList = general_method(workDirectory, data, node, useQueue, extendedProperties)

    log = logger.Logger().getLoggerUpload(node.id)
    if result and data:
        filesmodify.archive(log, workDirectory, node.backup_dir, maxCountFiles=maxCountBackupFiles, actualCountDays=actualCountDaysForBackupFiles)

    return result, errorList

def insertInDB(propertyReader, dictFile, extendedProperties):
    '''
    Подготавливает ноду, inserter для передачи в NES и загрузку в БД
    без использования очереди.
    Функция вызывается из sync агента напрямую
    '''
    from node import Node, AifConverterParameters, DBInserterConfig
    global maxCountBackupFiles
    global actualCountDaysForBackupFiles
    if os.path.split(sys.path[0])[1] == 'converter':
        sys.path[0] = os.path.normpath('%s/../..' % sys.path[0])
        sys.path.append(os.path.normpath('%s/upload/node' % sys.path[0]))

    shopCode = propertyReader.storeID
    log = logger.Logger().getLoggerUpload(shopCode)
    log.info(u'Загрузка данных напрямую в БД')
    log.info(u'Считываем конфигурацию')
    if not config.read(os.path.normpath(sys.path[0] + "/../config/cash/exchangers.ini")):
        log.error(u"Файл конфигурации'exchangers.ini' не найден.")
        sys.exit(1)

    backupDir =  macros.applyMacros(config.getString('UPLOAD', 'backupDir', '/linuxcash/cash/data/backup/dicts'), log, shopcode = shopCode)
    maxCountBackupFiles = config.getInt('UPLOAD', 'maxCountBackupFiles', 30)
    actualCountDaysForBackupFiles = config.getInt('UPLOAD', 'actualCountDaysForBackupFiles', 30)
    workDirectory = config.getString('main', 'workDirectory', '/linuxcash/cash/data/tmp/exchangers')

    pack_size = config.getInt('QUEUE', 'aifObjectsCountInMessage', 2000)
    if not os.path.exists(workDirectory):
        os.makedirs(workDirectory)

    flagName = dictFile + '.flag' if dictFile else None
    open(flagName, 'w+').close()
    node =  Node(id = shopCode,
            backup_dir = backupDir, 
            converters = [AifConverterParameters(flags = [flagName] if flagName else None, data = [dictFile] if dictFile else None)], 
            pack_size = pack_size,
            inserter = DBInserterConfig(),
            badDictsDirectory = macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/linuxcash/cash/data/backup/badDicts').strip(), log, shopcode = shopCode),
            extendedProperties = extendedProperties)

    node.currentConverter = node.converters[0]
    node.currentConverter.dict_state_id = extendedProperties.get('dictCashLoadStateId', None)
    node.cashCode = propertyReader.cashCode
    node.currentConverter.statusServer = '%s:%s' % (propertyReader.remoteIp, propertyReader.cashServerRestPort)
    log.info(u'Начинаем конвертирование')
    return convert(workDirectory, node, False, extendedProperties)

def main(argv):
    from node import Node, AifConverterParameters, QueueInserterConfig, QueueParameters
    global maxCountBackupFiles
    global actualCountDaysForBackupFiles

    parser = optparse.OptionParser()
    parser.add_option("-w", "--workDir", dest="workDir", default='/tmp/dictTasks',
                        help="Work directory for converter [default: %default]")
    parser.add_option("-i", "--storeId", dest="storeId", 
                        help="Shop code for CashServer mode")
    parser.add_option("-f", "--flagFiles", dest="flagFiles",
                           help="Array flag files", metavar="[FILE,..,FILE]")
    parser.add_option("-d", "--dictFiles", dest="dictFiles",
                           help="Array dictionary files", metavar="[FILE,..,FILE]")
    parser.add_option("-b", "--backupDir", dest="backupDir", default='',
                           help="Backup directory [default: %default]")
    parser.add_option("-s", "--packSize", dest="packSize", default=0, type='int',
                           help="Count of objects in one package [default: %default]")
    parser.add_option("-I", "--inserter", dest="inserter", default='queue', 
                        help="Loader for dictionaries objects [default: %default]")
    parser.add_option("-H", "--host", dest="host", default='', 
                           help="Host for inserter [default: %default]")
    parser.add_option("-p", "--port", dest="port", default='',
                           help="Port for inserter [default: %default]")
    parser.add_option("-l", "--login", dest="login", default='',
                           help="Login for inserter [default: %default]")
    parser.add_option("-P", "--passwd", dest="passwd", default='',
                           help="Password for inserter [default: %default]")
    parser.add_option("-D", "--dest", dest="dest", default='dictionaries',
                           help="Destination for inserter [default: %default]")
    (options, args) = parser.parse_args(argv)

    if options.storeId == None:
        parser.print_help()
        parser.error("specify storeId")
    if options.flagFiles == None:
        parser.print_help()
        parser.error("specify flagFiles")
    if options.dictFiles == None:
        parser.print_help()
        parser.error("specify dictFiles")

    flags = []
    flags.append(options.flagFiles)

    data = []
    data.append(options.dictFiles)

    logger.Logger('cash')
    log = logger.Logger().getLoggerUpload(options.storeId)
    if not config.read(os.path.normpath(sys.path[0] + "/../config/cash/exchangers.ini")):
        log.error(u"Файл конфигурации'exchangers.ini' не найден.")
        sys.exit(1)

    queue_host = ''
    queue_port = ''
    queue_user = ''
    queue_password = ''
    aifObjectsCountInMessage = 0

    if options.host == '':
        queue_host = config.getString('QUEUE', 'host', 'localhost')
    if options.port == '':
        queue_port = config.getInt('QUEUE', 'port', 5672)
    if options.login == '':
        queue_user = config.getString('QUEUE', 'user', 'guest')
    if options.passwd == '':
        queue_password = config.getString('QUEUE', 'password', 'guest')
    if options.packSize == 0:
        aifObjectsCountInMessage = config.getInt('QUEUE', 'aifObjectsCountInMessage', 200)
    backupDir =  options.backupDir if options.backupDir else macros.applyMacros(config.getString('UPLOAD', 'backupDir', '/linuxcash/cash/data/backup/dicts'), log, shopcode = options.storeId)
    maxCountBackupFiles = config.getInt('UPLOAD', 'maxCountBackupFiles', 30)
    actualCountDaysForBackupFiles = config.getInt('UPLOAD', 'actualCountDaysForBackupFiles', 30)

    node =  Node(id = options.storeId,
                backup_dir = backupDir, 
                converters = [AifConverterParameters(flags = flags, data = data)], 
                pack_size = aifObjectsCountInMessage, 
                inserter = QueueInserterConfig(queue = QueueParameters(host = queue_host, port = queue_port, username=queue_user, password=queue_password)),
                badDictsDirectory = macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/linuxcash/cash/data/backup/badDicts').strip(), log, shopcode = options.storeId))

    node.currentConverter = node.converters[0]
    node.currentConverter.dict_state_id = None

    convert(options.workDir, node)

if __name__ == '__main__':
    main(sys.argv[1:])
