# -*- coding: utf-8 -*-
from common import logger, filesmodify, config
from platform import node
log = logger.Logger().getLoggerUpload("soap")
from common import aifSoapClasses as soapCl
from common import aif_objects as aif
from upload.converter import soap
from common import rest_state, parameters
from upload.node.node import QueueInserterConfig, DBInserterConfig

import os
import time
from datetime import datetime
import inspect
import threading

from lxml import etree
from spyne import EventManager
from spyne.decorator import srpc
from spyne.model.complex import Array
from spyne.model.primitive import String, Boolean
from spyne.model.fault import Fault
from spyne.service import ServiceBase
from spyne.const.xml_ns import DEFAULT_NS
from spyne.const.http import HTTP_401
from spyne.server import wsgi

from spyne.application import Application
from spyne.protocol.soap import Soap11
from spyne.server.wsgi import WsgiApplication, WsgiMethodContext
from spyne.util.wsgi_wrapper import run_twisted
from wsgiref.simple_server import WSGIRequestHandler, ServerHandler, make_server
import traceback, sys
from base64 import b64encode

fileLogName = 'dict_SOAP_debug'

tns = 'http://aif.soap.exchange.artix.su'

primitiveAuth = False
soapLogin = 'login'
soapPasswd = 'passwd'
maxCountBackupRequests = 0
actualCountDaysForBackupRequests = 3
sessionIdleTimeout = 300

aifExchangeService = None


def getLogFileName(shopCode, dictionaryId, call_start):
    logDir = logger.Logger().getLogDirectory()
    return '%s/%s_%s_%s_%s.log' % (logDir, fileLogName, shopCode, dictionaryId, call_start)

class SoapConverter(object):
    name = 'soap'
    dict_state_id = None
    def __init__(self, hotKeyByBarcode = False):
        self.hotKeyByBarcode = hotKeyByBarcode

class EmptyNode(object):
    def __init__(self, shopId, realShopCode, isCashRun):
        self.id = shopId
        self.shopCode=realShopCode
        self.pack_size = config.getInt('QUEUE', 'aifObjectsCountInMessage', 200)
#        self.inserter = DBInserterConfig()
        self.inserter = QueueInserterConfig(queue = parameters.QueueParameters(host = config.getString('QUEUE', 'host', 'localhost'),
                                                                               port = config.getInt('QUEUE', 'port', 5672),
                                                                               username=config.getString('QUEUE', 'user', 'guest'),
                                                                               password=config.getString('QUEUE', 'password', 'guest'),
                                                                               queue='artix.loaddict.cash' if isCashRun else None))
        self.currentConverter = SoapConverter()
        self.extendedProperties = {}

class SoapDictSession(object):
    def __init__(self, node, dictId, converter):
        self.node = node
        self.dictId = dictId
        self.timeActive = time.time()
        self.converter = converter

class SessionId(object):
    def __init__(self, shopId, dictionaryId):
        self.shopId = shopId
        self.dictionaryId = dictionaryId

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return NotImplemented

    def __ne__(self, other):
        if isinstance(other, self.__class__):
            return not self.__eq__(other)
        return NotImplemented

    def __hash__(self):
        return hash(tuple(sorted(self.__dict__.items())))

class AifExchangeService(object):
    errorInCalling = 'Error at the stage of calling the load'

    timeoutNodeReread=300
    isCashRun=False
    shopCodeIsShopId=False
    useSession=False
    singleShopSession=True

    sessions = {}
    curNodes = []
    lastTimeReadNodes = 0

    def canOpenSession(self, shopId, dictionaryId):
        if self.singleShopSession:
            for sessionId in AifExchangeService.sessions.keys():
                if sessionId.shopId == shopId:
                    return False
            return True
        return not self.isOpenSession(shopId, dictionaryId)

    def isOpenSession(self, shopId, dictionaryId):
        return AifExchangeService.sessions.has_key(self.getSessionId(shopId, dictionaryId))

    def getSessionId(self, shopId, dictionaryId):
        return SessionId(shopId, dictionaryId)

    def rereadNodes(self):
        try:
            nodeGetter = __import__('upload.node.%s.nodeGetter' % (config.getString('UPLOAD', 'nodeGetter', 'cs30')), fromlist='q')
            nodeGetter.defaultNesServer=defaultNesServer
            nodeGetter.defaultStatusServer=defaultStatusServer
            AifExchangeService.curNodes = nodeGetter.getNodes()
            AifExchangeService.lastTimeReadNodes = time.time()
            log.info(u'ноды = %s' % ' '.join(str(e) for e in AifExchangeService.curNodes))
        except Exception, e:
            log.warning(u'Ошибка считывания нод. %s' % str(e))

    def checkShopCode(self, shopCode):
        if not shopCode or shopCode.strip() == '':
            return False, shopCode
        if time.time() - self.lastTimeReadNodes > self.timeoutNodeReread:
            self.rereadNodes()
        if self.curNodes:
            if not self.shopCodeIsShopId:
                ids = [n.id for n in self.curNodes if str(n.shopCode) == shopCode]
                if not ids:
                    log.warning(u'По коду %s не найдено ни одного магазина.' % shopCode)
                    return False, shopCode
                elif len(ids) == 1:
                    shopCode = ids[0]
                else:
                    log.warning(u'По коду %s найдено несколько магазинов. Невозможно однозначно определить целевой магазин.' % shopCode)
                    return False, shopCode
            elif not str(shopCode) in [str(node.id) for node in self.curNodes]:
                return False, shopCode
        elif not self.shopCodeIsShopId:
            log.warning(u'Невозможно найти по коду магазина его идентификатор, т.к. ноды не считаны.')
            return False, shopCode
        return True, shopCode

    def getPrestartResult(self, shopCode):
        dictConvertResult = soapCl.DictConvertResult()
        dictConvertResult.result = False
        dictConvertResult.info = self.errorInCalling
        res, shopId = self.checkShopCode(shopCode)
        if not res:
            dictConvertResult.info = 'Shop with code = %s does not exist.' % shopId
        return dictConvertResult, shopId

    def getNode(self, shopId, realShopCode, dictionaryId = None):
        node = EmptyNode(shopId, realShopCode, self.isCashRun)
        # nesServer и statusServer должен был бы задаться в nodeGetter, но тут используется своя EmptyNode
        node.currentConverter.statusServer = defaultStatusServer
        node.nesServer=defaultNesServer
        if not dictionaryId:
            dictionaryId = 'soap' + str(time.time())
        try:
            node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Start convert dict', u'Начало конвертирования справочника.', rest_state.STATUS_DURING, dictionaryId)
        except Exception,e:
            log.warning("Can't write state to CS by REST. %s" % e)
        for n in self.curNodes:
            if str(n.id) == str(node.id):
                node.inserter = n.inserter
                for converter in n.converters:
                    if converter.name == 'aif':
                        node.currentConverter.hotKeyByBarcode = converter.hotKeyByBarcode
                break
        return node

    def getMethodName(self, callerMethod):
        if callerMethod.startswith('add'):
            return 'addCmd'
        elif callerMethod.startswith('deleteByDate'):
            return 'deleteCmdByDate'
        elif callerMethod.startswith('deleteOverdue'):
            return 'deleteCmdOverdue'
        elif callerMethod.startswith('deleteOutdated'):
            return 'deleteCmdOutdated'
        elif callerMethod.startswith('deleteCatalogsAndTmcCatalogs'):
            return 'deleteCmdCatalogAndTmcCatalog'
        elif callerMethod.startswith('delete'):
            return 'deleteCmd'
        elif callerMethod.startswith('update'):
            return 'updateCmd'
        return

    def startSession(self, shopCode, dictionaryId):
        realShopCode=shopCode
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        if not self.useSession:
            result.result = False
            result.info = "Service is not configured for session mode."
            return result
        node = self.getNode(shopId, realShopCode, dictionaryId)
        if not self.canOpenSession(node.id, dictionaryId):
            result.info = 'Already opened session for shop %s' % (node.id)
            if not self.singleShopSession:
                result.info += 'and dictionaryId %s' % (dictionaryId)
            log.warning(result.info)
            return result

        log.info('Start session %s for dictionaryId %s' % (node.currentConverter.dict_state_id, dictionaryId))
        soapConverter = soap.SoapConverter(node)
        result = soapConverter.start()
        if result.result:
            AifExchangeService.sessions[self.getSessionId(node.id, dictionaryId)] = SoapDictSession(node, dictionaryId, soapConverter)
        return result

    def callCommand(self, aifClazz, soapObjects, shopCode = 'All', dictionaryId = None, clear=False, clearIfEmptyData=False):
        node = None
        soapConverter = None
        realShopCode = shopCode
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        if self.useSession:
            if not self.isOpenSession(shopId, dictionaryId):
                result.result = False
                result.info = "Can not add package. Session for shop %s and dictionaryId %s is not opened" % (realShopCode, dictionaryId)
                log.warning(result.info)
                return result
            log.info('Add package in session %s for dictionaryId %s' % (AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].node.currentConverter.dict_state_id, dictionaryId))
            result.result = True
            AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].timeActive = time.time()
            node = AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].node
            soapConverter = AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].converter
        else:
            node = self.getNode(shopId, realShopCode, dictionaryId)
            soapConverter = soap.SoapConverter(node)
            result = soapConverter.start()
            if not result.result:
                return result
            log.info('Loading dictionary id=%s' % (dictionaryId))
        result = soapConverter.convert(soapObjects, shopId, aifClazz, self.getMethodName(inspect.stack()[1][3]), clear, clearIfEmptyData) if result.result else result
        if not self.useSession:
            if result.result:
                result = soapConverter.stop()
            else:
                soapConverter.stop()
        return result

    def stopSession(self, shopCode, dictionaryId):
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        if not self.useSession:
            result.result = False
            result.info = "Service is not configured for session mode."
            return result
        if not self.isOpenSession(shopId, dictionaryId):
            result.info = 'Session for shopCode %s and dictionaryId %s is not open' % (shopCode, dictionaryId)
            log.warning(result.info)
            return result
        node = AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].node
        log.info('Stop session %s for shop %s and dictionaryId %s' % (node.currentConverter.dict_state_id, shopCode, dictionaryId))
        soapConverter = AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].converter
        result = soapConverter.stop()
        del AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)]
        return result

    def rollbackSession(self, shopCode, dictionaryId, sessionIdle=False):
        result, shopId = self.getPrestartResult(shopCode)
        if result.info != self.errorInCalling:
            log.info(result.info)
            return result
        if not self.useSession:
            result.result = False
            result.info = "Service is not configured for session mode."
            return result
        if not self.isOpenSession(shopId, dictionaryId):
            result.info = 'Session for shopCode %s and dictionaryId %s is not open' % (shopCode, dictionaryId)
            log.warning(result.info)
            return result
        node = AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].node
        log.info('Rollback session %s for shop %s and dictionaryId %s' % (node.currentConverter.dict_state_id, shopCode, dictionaryId))
        soapConverter = AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)].converter
        result = soapConverter.rollback(sessionIdle=sessionIdle)
        del AifExchangeService.sessions[self.getSessionId(shopId, dictionaryId)]
        return result

class AifSoapMethods(ServiceBase):
    __namespace__ = DEFAULT_NS

    """
    Список классов, наименования функций добавления и удаления
    для которых будет автоматически сгенерированы функции вставки и удаления по soap
    Классы в soap и aif должны называться одинаково.
    Например soapCl.Role, soapCl.RoleKey, aif.Role
    """
    listClasses = [
        ("Role", "addRoles", "deleteRoles"),
        ("SaleRestrict", "addSaleRestricts", "deleteSaleRestricts"),
        ("TmcRequiredQuantity","addTmcRequiredQuantities", "deleteTmcRequiredQuantities"),
        ("TmcVisualVerify", "addTmcVisualVerifies", "deleteTmcVisualVerifies"),
        ("Menu", "addMenu", "deleteMenu"),
        ("Cmaction", "addCmactions", "deleteCmactions"),
        ("CatalogAttribute", "addCatalogAttributes", "deleteCatalogAttributes"),
        ("Tmc", "addTmc", "deleteTmc")
        ]

    def createAddFunction(className, funcName):
        return """
@srpc(Array(soapCl.%s),
      String(default='All', min_occurs=1, nillable=False),
      Boolean(default=False),
      Boolean(default=False),
      String(encoding='utf8'),
      _returns=soapCl.DictConvertResult)
def %s(objects, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
    return aifExchangeService.callCommand(aif.%s, objects, shopCode, dictionaryId, clear, clearIfEmptyData)
        """ % (className, funcName, className)

    def createDeleteFunction(className, funcName):
        return """
@srpc(Array(soapCl.%sKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
def %s(objectKeys, shopCode = 'All', dictionaryId = None):
    return aifExchangeService.callCommand(aif.%s, objectKeys, shopCode, dictionaryId)
        """ % (className, funcName, className)

    # Создадим функции
    for className, addFuncName, deleteFuncName in listClasses:
        exec(createAddFunction(className, addFuncName))
        exec(createDeleteFunction(className, deleteFuncName))

    @srpc(String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def startSession(shopCode, dictionaryId):
        return aifExchangeService.startSession(shopCode, dictionaryId)

    @srpc(String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def stopSession(shopCode, dictionaryId):
        return aifExchangeService.stopSession(shopCode, dictionaryId)

    @srpc(String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def rollbackSession(shopCode, dictionaryId):
        return aifExchangeService.rollbackSession(shopCode, dictionaryId)

    @srpc(Array(soapCl.Accountpincode), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAccountpincodes(accountpincodes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Accountpincode, accountpincodes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AccountpincodeKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAccountpincodes(accountpincodeKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Accountpincode, accountpincodeKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Agerestrict), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAgerestricts(agerestricts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Agerestrict, agerestricts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AgerestrictKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAgerestricts(agerestrictKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Agerestrict, agerestrictKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Barcode), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addBarcodes(barcodes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Barcode, barcodes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.BarcodeKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteBarcodes(barcodeKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Barcode, barcodeKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Button), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addButtons(buttons, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Button, buttons, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ButtonKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteButtons(buttonKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Button, buttonKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Card), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCards(cards, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Card, cards, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CardKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCards(cardKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Card, cardKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CardGroup), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCardGroups(cardGroups, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroup, cardGroups, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CardGroupKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCardGroups(cardGroupKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroup, cardGroupKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CashUser), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCashUsers(cashUsers, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CashUser, cashUsers, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CashUserKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCashUsers(cashUserKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CashUser, cashUserKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Catalog), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCatalogs(catalogs, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Catalog, catalogs, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CatalogKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCatalogs(catalogKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Catalog, catalogKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CatalogKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCatalogsAndTmcCatalogs(catalogKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Catalog, catalogKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ChangeAccountBalance), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addChangeAccountBalances(changeAccountBalances, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeAccountBalance, changeAccountBalances, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ChangeAccountBalanceKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteChangeAccountBalances(changeAccountBalanceKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeAccountBalance, changeAccountBalanceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ChangeCardAccount), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addChangeCardAccounts(changeCardAccounts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeCardAccount, changeCardAccounts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ChangeCardAccountKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteChangeCardAccounts(changeCardAccountKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ChangeCardAccount, changeCardAccountKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Client), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addClients(clients, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Client, clients, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ClientKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteClients(clientKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Client, clientKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Organization), String(default = 'All', min_occurs = 1, nillable = False), Boolean(default = False), Boolean(default = False), String(encoding = 'utf8'), _returns=soapCl.DictConvertResult)
    def addOrganizations(organizations, shopCode = 'All', clear = False, clearIfEmptyData = False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Organization, organizations, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.OrganizationKey), String(default = 'All', min_occurs = 1, nillable = False), String(encoding = 'utf8'), _returns=soapCl.DictConvertResult)
    def deleteOrganizations(organizationKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Organization, organizationKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Command), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCommands(commands, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Command, commands, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CommandKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCommands(commandKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Command, commandKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Dept), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addDepts(depts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Dept, depts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.DeptKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteDepts(deptKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Dept, deptKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ExciseMark), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addExciseMarks(depts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMark, depts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ExciseMarkKey), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteExciseMarks(deptKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMark, deptKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ExciseMarkWhite), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addExciseMarkWhites(depts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMarkWhite, depts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ExciseMarkWhite), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteExciseMarkWhites(deptKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ExciseMarkWhite, deptKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.DiscountAction), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addDiscountActions(discountActions, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.DiscountAction, discountActions, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.DiscountActionKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteDiscountActions(discountActionKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.DiscountAction, discountActionKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.HotKey), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addHotKeys(hotKeys, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.HotKey, hotKeys, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.HotKeyKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteHotKeys(hotKeyKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.HotKey, hotKeyKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Invent), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addInvents(invents, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Invent, invents, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.InventKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteInvents(inventKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Invent, inventKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.InventCatalog), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addInventCatalogs(inventCatalogs, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.InventCatalog, inventCatalogs, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.InventCatalog), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteInventCatalogs(inventCatalogs, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.InventCatalog, inventCatalogs, shopCode, dictionaryId)

    @srpc(Array(soapCl.InventGroup), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addInventGroups(inventGroups, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.InventGroup, inventGroups, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.InventGroupKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteInventGroups(inventGroupKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.InventGroup, inventGroupKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.KbKey), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addKbKeys(kbKeys, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.KbKey, kbKeys, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.KbKeyKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteKbKeys(kbKeyKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.KbKey, kbKeyKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.LDiscAccumulate), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addLDiscAccumulates(lDiscAccumulates, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscAccumulate, lDiscAccumulates, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.LDiscAccumulateKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteLDiscAccumulates(lDiscAccumulateKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscAccumulate, lDiscAccumulateKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.LDiscCard), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addLDiscCards(lDiscCards, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscCard, lDiscCards, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.LDiscCardKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteLDiscCards(lDiscCardKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscCard, lDiscCardKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.LDiscFixed), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addLDiscFixeds(lDiscFixeds, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscFixed, lDiscFixeds, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.LDiscFixedKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteLDiscFixeds(lDiscFixedKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscFixed, lDiscFixedKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.LDiscScheme), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addLDiscSchemes(lDiscSchemes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscScheme, lDiscSchemes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.LDiscSchemeKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteLDiscSchemes(lDiscSchemeKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.LDiscScheme, lDiscSchemeKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.LVat), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addLVats(lVats, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.LVat, lVats, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.LVatKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteLVats(lVatKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.LVat, lVatKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.MCashUser), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addMCashUsers(mCashUsers, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.MCashUser, mCashUsers, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.MCashUserKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteMCashUsers(mCashUserKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.MCashUser, mCashUserKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Measure), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addMeasures(measures, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Measure, measures, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.MeasureKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteMeasures(measureKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Measure, measureKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Price), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addPrices(prices, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Price, prices, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.PriceKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deletePrices(pricesKeys, shopCode='All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Price, pricesKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.PriceKeyByDate), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteByDatePrices(priceKeysByDate, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Price, priceKeysByDate, shopCode, dictionaryId)

    @srpc(Array(soapCl.OverduePriceKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteOverduePrices(overduePriceKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Price, overduePriceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Remain), String(default='?', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addRemains(remains, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Remain, remains, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.RemainKey), String(default='?', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteRemains(remainsKeys, shopCode='All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Remain, remainsKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.OutdatedRemainKey), String(default='?', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteOutdatedRemains(outdatedRemainKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Remain, outdatedRemainKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Shop), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addShops(shops, shopCode = 'All', clear=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Shop, shops, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.ShopKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteShops(shopKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Shop, shopKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.TaxGroup), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTaxGroups(taxGroups, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TaxGroup, taxGroups, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TaxGroupKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTaxGroups(taxGroupKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TaxGroup, taxGroupKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Timerestrict), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTimerestricts(timerestricts, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Timerestrict, timerestricts, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TimerestrictKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTimerestricts(timerestrictKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Timerestrict, timerestrictKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Unit), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addUnits(units, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Unit, units, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.UnitKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteUnits(unitKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Unit, unitKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Currency), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCurrencies(currencies, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Currency, currencies, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CurrencyKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCurrencies(currencyKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Currency, currencyKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AdditionalPrice), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAdditionalPrices(additionalPrices, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.AdditionalPrice, additionalPrices, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AdditionalPriceKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAdditionalPrices(additionalPriceKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.AdditionalPrice, additionalPriceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.DiscountsAuto), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addDiscountsAutos(discountsAutos, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.DiscountsAuto, discountsAutos, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.DiscountsAutoKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteDiscountsAutos(discountsAutoKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.DiscountsAuto, discountsAutoKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.HotKeyInvent), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addHotKeyInvents(hotKeyInvents, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.HotKeyInvent, hotKeyInvents, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.HotKeyInvent), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteHotKeyInvents(hotKeyInvents, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.HotKeyInvent, hotKeyInvents, shopCode, dictionaryId)

    @srpc(Array(soapCl.Kit), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addKits(kits, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Kit, kits, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.KitKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteKits(kitKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Kit, kitKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.KitDisc), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addKitDiscs(kitDiscs, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.KitDisc, kitDiscs, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.KitDiscKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteKitDiscs(kitDiscKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.KitDisc, kitDiscKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.KitInvent), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addKitInvents(kitInvents, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.KitInvent, kitInvents, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.KitInventKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteKitInvents(kitInventKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.KitInvent, kitInventKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.TimePeriod), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTimePeriods(timePeriods, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TimePeriod, timePeriods, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TimePeriodKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTimePeriods(timePeriodKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TimePeriod, timePeriodKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Consultant), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addConsultants(consultants, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Consultant, consultants, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ConsultantKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteConsultants(consultantKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Consultant, consultantKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.BackReason), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addBackReasons(backReasons, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.BackReason, backReasons, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.BackReasonKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteBackReasons(backReasonKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.BackReason, backReasonKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CorrectionReason), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCorrectionReasons(correctionReason, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CorrectionReason, correctionReason, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CorrectionReasonKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCorrectionReasons(correctionReasonKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CorrectionReason, correctionReasonKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.SoftCheck), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addSoftChecks(softchecks, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.SoftCheck, softchecks, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.SoftCheckKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteSoftChecks(softcheckKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.SoftCheck, softcheckKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AlcoCode), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAlcoCodes(alcocodes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.AlcoCode, alcocodes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AlcoCode), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAlcoCodes(alcocodes, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.AlcoCode, alcocodes, shopCode, dictionaryId)

    @srpc(Array(soapCl.TmcAlcocode), String(default='All', min_occurs=1, nillable=False, values=['All']), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTmcAlcocodes(tmcAlcocodes, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcAlcocode, tmcAlcocodes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TmcAlcocode), String(default='All', min_occurs=1, nillable=False, values=['All']), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTmcAlcocodes(tmcAlcocodes, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcAlcocode, tmcAlcocodes, shopCode, dictionaryId)

    @srpc(Array(soapCl.TmcScale), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTmcScales(tmcScales, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcScale, tmcScales, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TmcScaleKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTmcScales(tmcScaleKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcScale, tmcScaleKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.TmcScaleGroup), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTmcScaleGroups(tmcScaleGroups, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcScaleGroup, tmcScaleGroups, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TmcScaleGroupKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTmcScaleGroups(tmcScaleGroupKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcScaleGroup, tmcScaleGroupKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Certificate), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCertificates(certificates, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Certificate, certificates, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CertificateKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCertificates(certificateKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Certificate, certificateKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Macro), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addMacros(marcos, shopCode='All', clear=False, clearIfEmptyData=False, dictionaryId=None):
        return aifExchangeService.callCommand(aif.Macro, marcos, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.MacroKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteMacros(macroKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Macro, macroKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Event), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addEvents(events, shopCode='All', clear=False, clearIfEmptyData=False, dictionaryId=None):
        return aifExchangeService.callCommand(aif.Event, events, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.EventKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteEvents(eventKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Event, eventKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Aspect), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAspects(aspects, shopCode='All', clear=False, clearIfEmptyData=False, dictionaryId=None):
        return aifExchangeService.callCommand(aif.Aspect, aspects, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AspectKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAspects(aspectKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.Aspect, aspectKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AspectScheme), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAspectSchemes(aspectschemes, shopCode='All', clear=False, clearIfEmptyData=False, dictionaryId=None):
        return aifExchangeService.callCommand(aif.AspectScheme, aspectschemes, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AspectSchemeKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAspectSchemes(aspectSchemeKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.AspectScheme, aspectSchemeKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AspectValue), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAspectValues(aspectvalues, shopCode='All', clear=False, clearIfEmptyData=False, dictionaryId=None):
        return aifExchangeService.callCommand(aif.AspectValue, aspectvalues, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AspectValueKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAspectValues(aspectValueKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.AspectValue, aspectValueKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AspectValueSet), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAspectValueSets(aspectvaluesets, shopCode='All', clear=False, clearIfEmptyData=False, dictionaryId=None):
        return aifExchangeService.callCommand(aif.AspectValueSet, aspectvaluesets, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AspectValueSetKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAspectValueSets(aspectValueSetKeys, shopCode='All', dictionaryId=None):
        return aifExchangeService.callCommand(aif.AspectValueSet, aspectValueSetKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AspectValueSetValues), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAspectValueSetValues(aspectvaluesetvalues, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.AspectValueSetValues, aspectvaluesetvalues, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AspectValueSetValues), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAspectValueSetValues(aspectvaluesetvalues, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.AspectValueSetValues, aspectvaluesetvalues, shopCode, dictionaryId)

    @srpc(Array(soapCl.AlcoSet), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAlcoSet(alcoset, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.AlcoSet, alcoset, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AlcoSetKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAlcoSet(alcosetKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.AlcoSet, alcosetKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AdditionalCard), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addAdditionalCard(additionalcard, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.AdditionalCard, additionalcard, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.AdditionalCardKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteAdditionalCard(additionalcardKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.AdditionalCard, additionalcardKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CardGroupSearchPlace), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCardGroupSearchPlace(cardgroupsearchplace, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroupSearchPlace, cardgroupsearchplace, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CardGroupSearchPlaceKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCardGroupSearchPlace(cardgroupsearchplaceKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroupSearchPlace, cardgroupsearchplaceKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Verification), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addVerifications(verifications, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Verification, verifications, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.VerificationKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteVerifications(verificationKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Verification, verificationKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.CardGroupVerification), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCardGroupVerifications(cardgroupverifications, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroupVerification, cardgroupverifications, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CardGroupVerification), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCardGroupVerifications(cardgroupverifications, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CardGroupVerification, cardgroupverifications, shopCode, dictionaryId)

    @srpc(Array(soapCl.CashConfig), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCashConfig(cashconfig, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.CashConfig, cashconfig, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CashConfigKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCashConfig(cashconfigKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.CashConfig, cashconfigKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.SecurityWeight), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addSecurityWeight(securityWeight, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.SecurityWeight, securityWeight, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.SecurityWeightKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteSecurityWeight(securityWeightKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.SecurityWeight, securityWeightKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ActionPanel), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addActionPanel(actionPanel, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.ActionPanel, actionPanel, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.ActionPanelKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteActionPanel(actionPanelKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.ActionPanel, actionPanelKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Country), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addCountry(country, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Country, country, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.CountryKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteCountry(countryKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Country, countryKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Supplier), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addSuppliers(suppliers, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Supplier, suppliers, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.SupplierKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteSuppliers(supplierKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Supplier, supplierKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.TmcSupplier), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addTmcSuppliers(tmcsuppliers, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcSupplier, tmcsuppliers, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.TmcSupplier), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteTmcSuppliers(tmcsuppliers, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.TmcSupplier, tmcsuppliers, shopCode, dictionaryId)

    @srpc(Array(soapCl.MCashUserAccess), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addMCashUserAccesses(mcashuseraccesses, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.MCashUserAccess, mcashuseraccesses, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.MCashUserAccess), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deleteMCashUserAccesses(mcashuseraccesses, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.MCashUserAccess, mcashuseraccesses, shopCode, dictionaryId)

    @srpc(Array(soapCl.Picklist), String(default='All', min_occurs=1, nillable=False), Boolean(default=False), Boolean(default=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def addPicklists(picklists, shopCode = 'All', clear=False, clearIfEmptyData=False, dictionaryId = None):
        return aifExchangeService.callCommand(aif.Picklist, picklists, shopCode, dictionaryId, clear, clearIfEmptyData)

    @srpc(Array(soapCl.PicklistKey), String(default='All', min_occurs=1, nillable=False), String(encoding='utf8'), _returns=soapCl.DictConvertResult)
    def deletePicklists(picklistKeys, shopCode = 'All', dictionaryId = None):
        return aifExchangeService.callCommand(aif.Picklist, picklistKeys, shopCode, dictionaryId)

class AifEventManager(EventManager):
    def getLogName(self, ctx):
        if ctx and ctx.in_object:
            return getLogFileName(ctx.in_object.shopCode, ctx.in_object.dictionaryId, ctx.call_start)
        elif ctx:
            return getLogFileName('UNCKNOWN_SHOP', 'UNCKNOWN_DICT_ID', ctx.call_start)
        else:
            return getLogFileName('UNCKNOWN_SHOP', 'UNCKNOWN_DICT_ID', time.time()*1000)

    def fire_event(self, event_name, ctx):
        if event_name=='aif_log_request':
            with open(self.getLogName(ctx), 'a') as f:
                f.write('%s Request:\n' % datetime.now())
                f.write(etree.tostring(ctx.in_document, pretty_print=True))
        if event_name=='wsgi_close':
            logFile = self.getLogName(ctx)
            with open(logFile, 'a') as f:
                f.write('%s Response:\n' % datetime.now())
                f.write(etree.tostring(ctx.out_document, pretty_print=True))
            if ctx and ctx.in_object:
                backupRequest(ctx.in_object.shopCode, [logFile])
            else:
                backupRequest('UNCKNOWN_SHOP', [logFile])
        EventManager.fire_event(self, event_name, ctx)

class AifWsgiApplication(WsgiApplication):
    def __init__(self, app, chunked=True, max_content_length=2 * 1024 * 1024, block_length=8 * 1024):
        super(AifWsgiApplication, self).__init__(app, chunked, max_content_length, block_length)
        self.event_manager = AifEventManager(self)
        if self.doc.wsdl11 is not None:
            self.doc.wsdl11.event_manager.add_listener("wsdl_document_built", self.on_wsdl_document_built)

    # ставим тег с сервисами в самый конец, чтобы формировалась корректная wsdl
    def on_wsdl_document_built(self, doc):
        services = doc.root_elt.findall('{http://schemas.xmlsoap.org/wsdl/}service')
        for service in services:
            doc.root_elt.remove(service)
            doc.root_elt.append(service)

    def __call__(self, req_env, start_response, wsgi_url=None):
        print req_env
        if primitiveAuth:
            userAndPass = b64encode(b"%s:%s" % (soapLogin, soapPasswd)).decode("ascii")
            if req_env.get('HTTP_AUTHORIZATION') != 'Basic %s' %  userAndPass:
                ctx = WsgiMethodContext(self, req_env, 'text/xml; charset=utf-8')
                start_response(HTTP_401, wsgi._gen_http_headers(ctx.transport.resp_headers))
                return [HTTP_401]
        return WsgiApplication.__call__(self, req_env, start_response, wsgi_url=None)


class AifApplication(Application):
    def __init__(self, services, tns, name=None, in_protocol=None, out_protocol=None, interface=None):
        Application.__init__(self, services, tns, name, in_protocol, out_protocol, interface)
        self.event_manager = AifEventManager(self)

    def process_request(self, ctx):
        self.event_manager.fire_event('aif_log_request', ctx)
        Application.process_request(self, ctx)

    def call_wrapper(self, ctx):
        try:
            return Application.call_wrapper(self, ctx)
        except Exception, e:
            raise Fault(faultstring=e.message)


class ServerHandlerAif(ServerHandler):
    def error_output(self, environ, start_response):
        try:
            start_response(self.error_status,self.error_headers[:],sys.exc_info())
            exc_type, exc_value, exc_traceback = sys.exc_info()
            mess = traceback.format_exception_only(exc_type, exc_value)
        except Exception, e:
            print e
        result = []
        result.append('''<?xml version='1.0' encoding='UTF-8'?>''')
        result.append('''<senv:Envelope xmlns:tns="%s" xmlns:senv="http://schemas.xmlsoap.org/soap/envelope/"><senv:Body><tns:Response><tns:Result><tns:info>%s</tns:info><tns:result>false</tns:result></tns:Result></tns:Response></senv:Body></senv:Envelope>''' % (tns, mess))

        return result


class WSGIRequestHandlerAif(WSGIRequestHandler):

    timeout = 300 # connection timeout

    def handle(self):
        self.raw_requestline = self.rfile.readline()
        print self.raw_requestline
        if not self.parse_request(): # An error code has been sent, just exit
            return
        if primitiveAuth:
            userAndPass = b64encode(b"%s:%s" % (soapLogin, soapPasswd)).decode("ascii")
            if self.headers.getheader('Authorization') != 'Basic %s' %  userAndPass:
                self.send_response(401, message='Authorization error.')
                return
        handler = ServerHandlerAif(
            self.rfile, self.wfile, self.get_stderr(), self.get_environ()
        )
        handler.request_handler = self
        handler.run(self.server.get_app())

def backupRequest(shopCode, logFiles=None):
    logDir = logger.Logger().getLogDirectory()
    if not logFiles:
        logFiles = filesmodify.getFilesByMask(logDir, ['%s_%s*' % (fileLogName, shopCode)])
    for f in logFiles:
        filesmodify.archive(log, os.path.dirname(f), os.path.join(logDir, 'SOAP_BACKUP', shopCode),
                            maxCountFiles=maxCountBackupRequests, actualCountDays=actualCountDaysForBackupRequests,
                            removeSRC = True, fileName = os.path.basename(f), loginfo=False)

def sessionThread():
    try:
        while True:
            sessions = AifExchangeService.sessions
            for k in list(sessions):
                if (time.time() - sessions[k].timeActive) > sessionIdleTimeout:
                    aifExchangeService.rollbackSession(sessions[k].node.id, sessions[k].dictId, sessionIdle=True)
            time.sleep(2)
    except Exception, e:
        log.error(e)

def start(isCash = False):
    global primitiveAuth
    global soapLogin
    global soapPasswd
    global maxCountBackupRequests
    global actualCountDaysForBackupRequests
    global sessionIdleTimeout
    global aifExchangeService
    global defaultStatusServer
    global defaultNesServer

    AifExchangeService.timeoutNodeReread=config.getInt('UPLOAD', 'timeoutNodeReread', 300)
    AifExchangeService.isCashRun=isCash
    AifExchangeService.shopCodeIsShopId=config.getBool('SOAP_UPLOAD', 'shopCodeIsShopId', False)
    AifExchangeService.useSession=config.getBool('SOAP_UPLOAD', 'useSession', False)
    AifExchangeService.singleShopSession=config.getBool('SOAP_UPLOAD', 'singleShopSession', False)

    useValidation = config.getBool('SOAP_UPLOAD', 'useValidation', False)

    aifExchangeService = AifExchangeService()

    maxContentLength = config.getInt('SOAP_UPLOAD', 'maxContentLength', (8*1024*1024)*10)
    primitiveAuth = config.getBool('SOAP_UPLOAD', 'authorize', False)
    soapLogin = config.getString('SOAP_UPLOAD', 'login', 'admin')
    soapPasswd = config.getString('SOAP_UPLOAD', 'passwd', 'admin')
    maxCountBackupRequests = config.getInt('SOAP_UPLOAD', 'maxCountBackupRequests', 0)
    actualCountDaysForBackupRequests = config.getInt('SOAP_UPLOAD', 'actualCountDaysForBackupRequests', 3)
    sessionIdleTimeout=config.getInt('SOAP_UPLOAD', 'sessionIdleTimeout', 300)

    port = config.getInt('SOAP_UPLOAD', 'port', 18080)
    #NEW
    multiprocessing = config.getBool('SOAP_UPLOAD', 'multiprocessing', False)

    defaultStatusServer = config.getString('CS', 'defaultStatusServer', 'localhost:8080')
    defaultNesServer= config.getString('CS', 'defaultNesServer', 'localhost:28081')

    blockLength=10*1024*1024 # 10 MB
    if blockLength > maxContentLength:
        blockLength = maxContentLength



    if AifExchangeService.useSession:
        log.info("Starting thread for idle session")
        thread = threading.Thread(target=sessionThread)
        thread.daemon = True
        thread.start()

    if multiprocessing:
        log.info("Starting multiprocessing: exchangeUploadSoapService")
        application = AifApplication([AifSoapMethods], tns, name = 'AifExchange', in_protocol=Soap11(validator='soft' if useValidation else None), out_protocol=Soap11())
        twisted_apps = [
            (AifWsgiApplication(application, chunked=True, max_content_length=maxContentLength, block_length=blockLength), 'soap'),
        ]
        run_twisted(twisted_apps, port)
    else:
        log.info("Starting single process exchangeUploadSoapService")
        application = AifApplication([AifSoapMethods], tns, name = 'AifExchange', in_protocol=Soap11(validator='soft' if useValidation else None), out_protocol=Soap11())
        wsgi_app = AifWsgiApplication(application, max_content_length=maxContentLength, block_length=blockLength)
        server = make_server('0.0.0.0', port, wsgi_app, handler_class=WSGIRequestHandlerAif)
        server.serve_forever()

    log.info("Stoped exchangeUploadSoapService")

