# -*- coding: utf-8 -*-

from upload.node.node import Node, QueueInserterConfig, QueueParameters, UsersConverterParameters
import registry
import tempfile
import os, sys
from common import macros, logger, config
from common.cashconvertersconfig import CashConvertersConfig
from upload.node.convertergetter import getConverterByName

registryPath = '/linuxcash/cash/data/cash.reg'
reg = registry.Registry(registryPath)
log = logger.Logger().getLoggerUpload("main")
badDictsDirectory = '%s/exchangers/badDicts' % tempfile.gettempdir()

shopCode = str(reg.shopcode())
cashCode = str(reg.cashcode())

def readStandartParams(section):
    flags = [ macros.applyMacros(fl.strip(), log, shopcode = shopCode, cashcode = cashCode) for fl in config.getString(section, 'flags', '/tmp/dict/aif/pos*.flz').split(',') ]
    data = [ macros.applyMacros(d.strip(), log, shopcode = shopCode, cashcode = cashCode) for d in config.getString(section, 'data', '/tmp/dict/aif/pos*.aif').split(',') ]
    errorDest = macros.applyMacros(config.getString(section, 'errorDest', '').strip(), log, shopcode = shopCode, cashcode = cashCode)
    return flags, data, errorDest

def getConverter(converterName):
    converter = None
    section = 'upload.converter.%s' % converterName
    flags, data, errorDest = readStandartParams(section)

    if (converterName == 'users'):
        converter = UsersConverterParameters(flags=flags, data=data, errorDest=errorDest)
    else:
        convertersConfig = CashConvertersConfig(config, section)
        converter = getConverterByName(converterName=converterName, convertersConfig=convertersConfig, flags=flags, data=data, errorDest=errorDest)
    return converter

def getNodes():
    log.info(u"Получение ноды")

    configPath = os.path.normpath(sys.path[0] + "/../config/cash")
    configFiles = [configPath + "/exchangers.ini"]
    configPath += "/conf.d/"
    if os.path.isdir(configPath):
        for item in os.listdir(configPath):
            if os.path.isfile(configPath + item) and item.endswith(".ini"):
                configFiles.append(configPath + item)
    readFiles = config.read(configFiles)
    if not readFiles == configFiles:
        for fileName in configFiles:
            if fileName not in readFiles:
                log.error(u"Невозможно открыть конфигурационный файл \"%s\"" % fileName)
        sys.exit(1)

    if not shopCode or shopCode == '':
        log.error(u"Ошибка получения кода кассы из %s" % registryPath)
        sys.exit(1)

    queue_host = config.getString('QUEUE', 'host', 'localhost')
    queue_port = config.getInt('QUEUE', 'port', 5672)
    queue_user = config.getString('QUEUE', 'user', 'guest')
    queue_password = config.getString('QUEUE', 'password', 'guest')
    aifObjectsCountInMessage = config.getInt('QUEUE', 'aifObjectsCountInMessage', 200)

    badDictsDirectory = macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/linuxcash/cash/data/backup/badDicts').strip(), log, shopcode = shopCode, cashcode = cashCode)
    convertersInConf = config.getString('UPLOAD', 'converter', 'aif').split(',')
    converters = []
    for converterName in convertersInConf:
        converterName = converterName.strip().lower()
        converter = getConverter(converterName)
        if converter:
            converters.append(converter)
            log.info(u'Конвертер - \'%s\'' % converterName)
            log.info(u'Файлы данных - \'%s\'' % converter.data)
            log.info(u'Файлы флагов - \'%s\'' % converter.flags)
            log.info(u'Файл информации об ошибках - \'%s\'' % converter.errorDest)

    nodes = []
    nodes.append(
                Node(id = shopCode,
                    backup_dir = macros.applyMacros(config.getString('UPLOAD', 'backupDir', '/linuxcash/cash/data/backup/dicts'), log, shopcode = shopCode, cashcode = cashCode),
                    converters = converters,
                    pack_size = aifObjectsCountInMessage,
                    inserter = QueueInserterConfig(queue = QueueParameters(host = queue_host, port = queue_port, username=queue_user, password=queue_password, queue='artix.loaddict.cash')),
                    badDictsDirectory = badDictsDirectory)
                )
    log.info(u"Получение ноды завершено")
    return nodes
