# -*- coding: utf-8 -*-

from upload.node.node import AifConverterParameters, SmagConverterParameters, SmagUkm4ConverterParameters, OracleConverterParameters, ShtrihConverterParameters, AtolConverterParameters, CrystalConverterParameters, SapConverterParameters, Cif1ConverterParameters

def getConverterByName(converterName, convertersConfig, flags, data, errorDest=None, backupDir=None):
    converter = None

    if converterName in ['aif', 'ARTIXINTERCHANGE']:
       hotKeyByBarcode = convertersConfig.getBool('hotKeyByBarcode', False)
       checkMatchFileNames = convertersConfig.getBool('checkMatchFileNames', False)
       converter = AifConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir, hotKeyByBarcode=hotKeyByBarcode, checkMatchFileNames=checkMatchFileNames)
    elif converterName in ['shtrih', 'SHTRIHM']:
        delimiter = convertersConfig.getString('delimiter', ';')
        cashierName = convertersConfig.getBool('cashierName', False)
        hotKeyByBarcode = convertersConfig.getBool('hotKeyByBarcode', False)
        requireSaleRestrict = convertersConfig.getBool('requireSaleRestrict', False)
        converter = ShtrihConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir, delimiter=delimiter,
                                              cashierName=cashierName, hotKeyByBarcode=hotKeyByBarcode, requireSaleRestrict=requireSaleRestrict)
    elif converterName in ['atol', 'ATOL']:
        useMultiplicityQuantityForMeasure = convertersConfig.getBool('useMultiplicityQuantityForMeasure', False)
        converter = AtolConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir,
                                            useMultiplicityQuantityForMeasure=useMultiplicityQuantityForMeasure)
    elif converterName in ['oracle', 'ORACLE']:
        converter = OracleConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir)
    elif converterName in ['smag-ukm', 'smag-ukm4', 'SUPERMAG', 'SUPERMAG4']:
        charset = convertersConfig.getString('charset', '866')
        format = convertersConfig.getString('format', 'dat')
        defaultRoleCode = convertersConfig.getInt('defaultRoleCode', 3)
        additionallyRoleCode = convertersConfig.getInt('additionallyRoleCode', 2)
        usersWithAdditionallyRole = convertersConfig.getString('usersWithAdditionallyRole', '')
        relationRolesBy = convertersConfig.getString('relationRolesBy', 'name')
        substringInAdminName = convertersConfig.getString('substringInAdminName', 'Администратор')
        catalogBindCode = convertersConfig.getString('catalogBindCode', '')
        groupBindCode = convertersConfig.getString('groupBindCode', '')
        setMinPriceIfDiscountRestrictMissing = convertersConfig.getBool('setMinPriceIfDiscountRestrictMissing', True)
        group2CatalogMapper = convertersConfig.getGroup2CatalogMapper()
        if converterName in ['smag-ukm', 'SUPERMAG']:
            converter = SmagConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir, charset=charset, format=format,
                                                defaultRoleCode=defaultRoleCode, additionallyRoleCode=additionallyRoleCode,
                                                usersWithAdditionallyRole=usersWithAdditionallyRole, relationRolesBy=relationRolesBy,
                                                substringInAdminName=substringInAdminName, catalogBindCode=catalogBindCode,
                                                groupBindCode=groupBindCode, setMinPriceIfDiscountRestrictMissing=setMinPriceIfDiscountRestrictMissing,
                                                group2CatalogMapper=group2CatalogMapper)
        else:
            discount = convertersConfig.getString('discount', 'modern')
            usersWithAdditionallyRole = usersWithAdditionallyRole.split(',')
            converter = SmagUkm4ConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir, charset=charset,
                                                    format=format, discount=discount, defaultRoleCode=defaultRoleCode,
                                                    additionallyRoleCode=additionallyRoleCode, usersWithAdditionallyRole=usersWithAdditionallyRole,
                                                    relationRolesBy=relationRolesBy, substringInAdminName=substringInAdminName,
                                                    catalogBindCode=catalogBindCode, groupBindCode=groupBindCode,
                                                    group2CatalogMapper=group2CatalogMapper)
    elif converterName in ['crystal', 'CRYSTAL']:
        visualVerifySG = convertersConfig.getString('visualverifysg', '1,2,3')
        requireQuantityManualSG = convertersConfig.getString('requirequantitymanualsg', '4,5,6')
        measureMapper = convertersConfig.getParamValue('measuremap', 'шт-1,1007-2')
        if not isinstance(measureMapper, unicode):
            measureMapper = unicode(measureMapper, 'utf-8')
        converter = CrystalConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir,
                                               visualVerifySG=visualVerifySG, requireQuantityManualSG=requireQuantityManualSG,
                                               measureMapper=measureMapper)
    elif converterName in ['sap', 'SAP']:
        converter = SapConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir)
    elif converterName in ['cif1', 'CIF1']:
        converter = Cif1ConverterParameters(flags=flags, data=data, errorDest=errorDest, backup_dir=backupDir)
    return converter
